@@ -2,6 +2,7 @@
 
 """
 import os
+import ssl
 import sys
 import re
 import errno
@@ -103,7 +104,21 @@ OUTPUT_OPTIONS_DEFAULT = OUT_RESP_HEAD + OUT_RESP_BODY
 OUTPUT_OPTIONS_DEFAULT_STDOUT_REDIRECTED = OUT_RESP_BODY
 
 
-class Parser(ArgumentParser):
+SSL_VERSION_ARG_MAPPING = {
+    'ssl2.3': 'PROTOCOL_SSLv23',
+    'ssl3': 'PROTOCOL_SSLv3',
+    'tls1': 'PROTOCOL_TLSv1',
+    'tls1.1': 'PROTOCOL_TLSv1_1',
+    'tls1.2': 'PROTOCOL_TLSv1_2',
+}
+SSL_VERSION_ARG_MAPPING = dict(
+    (cli_arg, getattr(ssl, ssl_constant))
+    for cli_arg, ssl_constant in SSL_VERSION_ARG_MAPPING.items()
+    if hasattr(ssl, ssl_constant)
+)
+
+
+class HTTPieArgumentParser(ArgumentParser):
     """Adds additional logic to `argparse.ArgumentParser`.
 
     Handles all input (CLI args, file args, stdin), applies defaults,
@@ -113,13 +128,13 @@ class Parser(ArgumentParser):
 
     def __init__(self, *args, **kwargs):
         kwargs['add_help'] = False
-        super(Parser, self).__init__(*args, **kwargs)
+        super(HTTPieArgumentParser, self).__init__(*args, **kwargs)
 
     # noinspection PyMethodOverriding
     def parse_args(self, env, args=None, namespace=None):
 
         self.env = env
-        self.args, no_options = super(Parser, self)\
+        self.args, no_options = super(HTTPieArgumentParser, self)\
             .parse_known_args(args, namespace)
 
         if self.args.debug:
@@ -127,7 +142,6 @@ class Parser(ArgumentParser):
 
         # Arguments processing and environment setup.
         self._apply_no_options(no_options)
-        self._apply_config()
         self._validate_download_options()
         self._setup_standard_streams()
         self._process_output_options()
@@ -164,19 +178,17 @@ class Parser(ArgumentParser):
         }.get(file, file)
         if not hasattr(file, 'buffer') and isinstance(message, str):
             message = message.encode(self.env.stdout_encoding)
-        super(Parser, self)._print_message(message, file)
+        super(HTTPieArgumentParser, self)._print_message(message, file)
 
     def _setup_standard_streams(self):
         """
         Modify `env.stdout` and `env.stdout_isatty` based on args, if needed.
 
         """
-        if not self.env.stdout_isatty and self.args.output_file:
-            self.error('Cannot use --output, -o with redirected output.')
-
+        self.args.output_file_specified = bool(self.args.output_file)
         if self.args.download:
             # FIXME: Come up with a cleaner solution.
-            if not self.env.stdout_isatty:
+            if not self.args.output_file and not self.env.stdout_isatty:
                 # Use stdout as the download output file.
                 self.args.output_file = self.env.stdout
             # With `--download`, we write everything that would normally go to
@@ -201,11 +213,6 @@ class Parser(ArgumentParser):
             self.env.stdout = self.args.output_file
             self.env.stdout_isatty = False
 
-    def _apply_config(self):
-        if (not self.args.json
-                and self.env.config.implicit_content_type == 'form'):
-            self.args.form = True
-
     def _process_auth(self):
         """
         If only a username provided via --auth, then ask for a password.
@@ -336,17 +343,14 @@ class Parser(ArgumentParser):
                     'Invalid file fields (perhaps you meant --form?): %s'
                     % ','.join(file_fields))
 
-            fn, fd = self.args.files['']
+            fn, fd, ct = self.args.files['']
             self.args.files = {}
 
             self._body_from_file(fd)
 
             if 'Content-Type' not in self.args.headers:
-                mime, encoding = mimetypes.guess_type(fn, strict=False)
-                if mime:
-                    content_type = mime
-                    if encoding:
-                        content_type = '%s; charset=%s' % (mime, encoding)
+                content_type = get_content_type(fn)
+                if content_type:
                     self.args.headers['Content-Type'] = content_type
 
     def _process_output_options(self):
@@ -355,18 +359,32 @@ class Parser(ArgumentParser):
         The default output options are stdout-type-sensitive.
 
         """
-        if not self.args.output_options:
-            self.args.output_options = (
-                OUTPUT_OPTIONS_DEFAULT
-                if self.env.stdout_isatty
-                else OUTPUT_OPTIONS_DEFAULT_STDOUT_REDIRECTED
-            )
+        def check_options(value, option):
+            unknown = set(value) - OUTPUT_OPTIONS
+            if unknown:
+                self.error('Unknown output options: {0}={1}'.format(
+                    option,
+                    ','.join(unknown)
+                ))
 
-        unknown_output_options = set(self.args.output_options) - OUTPUT_OPTIONS
-        if unknown_output_options:
-            self.error(
-                'Unknown output options: %s' % ','.join(unknown_output_options)
-            )
+        if self.args.verbose:
+            self.args.all = True
+
+        if self.args.output_options is None:
+            if self.args.verbose:
+                self.args.output_options = ''.join(OUTPUT_OPTIONS)
+            else:
+                self.args.output_options = (
+                    OUTPUT_OPTIONS_DEFAULT
+                    if self.env.stdout_isatty
+                    else OUTPUT_OPTIONS_DEFAULT_STDOUT_REDIRECTED
+                )
+
+        if self.args.output_options_history is None:
+            self.args.output_options_history = self.args.output_options
+
+        check_options(self.args.output_options, '--print')
+        check_options(self.args.output_options_history, '--history-print')
 
         if self.args.download and OUT_RESP_BODY in self.args.output_options:
             # Response body is always downloaded with --download and it goes
@@ -609,6 +627,21 @@ RequestItems = namedtuple('RequestItems',
                           ['headers', 'data', 'files', 'params'])
 
 
+def get_content_type(filename):
+    """
+    Return the content type for ``filename`` in format appropriate
+    for Content-Type headers, or ``None`` if the file type is unknown
+    to ``mimetypes``.
+
+    """
+    mime, encoding = mimetypes.guess_type(filename, strict=False)
+    if mime:
+        content_type = mime
+        if encoding:
+            content_type = '%s; charset=%s' % (mime, encoding)
+        return content_type
+
+
 def parse_items(items,
                 headers_class=CaseInsensitiveDict,
                 data_class=OrderedDict,
@@ -634,7 +667,8 @@ def parse_items(items,
             try:
                 with open(os.path.expanduser(value), 'rb') as f:
                     value = (os.path.basename(value),
-                             BytesIO(f.read()))
+                             BytesIO(f.read()),
+                             get_content_type(value))
             except IOError as e:
                 raise ParseError('"%s": %s' % (item.orig, e))
             target = files
