@@ -10,6 +10,22 @@ def load_json_preserve_order(s):
     return json.loads(s, object_pairs_hook=OrderedDict)
 
 
+def repr_dict_nice(d):
+    def prepare_dict(d):
+        for k, v in d.items():
+            if isinstance(v, dict):
+                v = dict(prepare_dict(v))
+            elif isinstance(v, bytes):
+                v = v.decode('utf8')
+            elif not isinstance(v, (int, str)):
+                v = repr(v)
+            yield k, v
+    return json.dumps(
+        dict(prepare_dict(d)),
+        indent=4, sort_keys=True,
+    )
+
+
 def humanize_bytes(n, precision=2):
     # Author: Doug Latornell
     # Licence: MIT
